@@ -5,8 +5,8 @@ use warnings;
 use 5.008001;
 use Carp qw( croak );
 
-# ABSTRACT: Write Perl bindings to non-Perl libraries without C or XS
-our $VERSION = '0.19'; # VERSION
+# ABSTRACT: Write Perl bindings to non-Perl libraries with FFI. No XS required.
+our $VERSION = '0.20'; # VERSION
 
 # Platypus Man,
 # Platypus Man,
@@ -271,8 +271,13 @@ sub function
 }
 
 
+my $inner_counter=0;
+
 sub attach
 {
+  my $wrapper;
+  $wrapper = pop if ref $_[-1] eq 'CODE';
+
   my($self, $name, $args, $ret, $proto) = @_;
   my($c_name, $perl_name) = ref($name) ? @$name : ($name, $name);
 
@@ -287,7 +292,22 @@ sub attach
     $perl_name = join '::', $caller, $perl_name
       unless $perl_name =~ /::/;
     
-    $function->attach($perl_name, "$filename:$line", $proto);
+    my $attach_name = $perl_name;
+    if($wrapper)
+    {
+      $attach_name = "FFI::Platypus::Inner::xsub$inner_counter";
+      $inner_counter++;
+    }
+    
+    $function->attach($attach_name, "$filename:$line", $proto);
+    
+    if($wrapper)
+    {
+      my $inner_coderef = \&{$attach_name};
+      no strict 'refs';
+      # TODO: Sub::Name ?
+      *{$perl_name} = sub { $wrapper->($inner_coderef, @_) };
+    }
   }
   
   $self;
@@ -325,6 +345,15 @@ sub sizeof
 }
 
 
+sub find_lib
+{
+  my $self = shift;
+  require FFI::CheckLib;
+  $self->lib(FFI::CheckLib::find_lib(@_));
+  $self;
+}
+
+
 sub find_symbol
 {
   my($self, $name) = @_;
@@ -418,7 +447,7 @@ sub DESTROY
 
 package FFI::Platypus::Function;
 
-our $VERSION = '0.19'; # VERSION
+our $VERSION = '0.20'; # VERSION
 
 use overload '&{}' => sub {
   my $ffi = shift;
@@ -430,7 +459,7 @@ package FFI::Platypus::Closure;
 use Scalar::Util qw( refaddr);
 use Carp qw( croak );
 
-our $VERSION = '0.19'; # VERSION
+our $VERSION = '0.20'; # VERSION
 
 our %cbdata;
 
@@ -457,13 +486,13 @@ sub DESTROY
 
 package FFI::Platypus::ClosureData;
 
-our $VERSION = '0.19'; # VERSION
+our $VERSION = '0.20'; # VERSION
 
 package FFI::Platypus::Type;
 
 use Carp qw( croak );
 
-our $VERSION = '0.19'; # VERSION
+our $VERSION = '0.20'; # VERSION
 
 sub new
 {
@@ -549,11 +578,11 @@ __END__
 
 =head1 NAME
 
-FFI::Platypus - Write Perl bindings to non-Perl libraries without C or XS
+FFI::Platypus - Write Perl bindings to non-Perl libraries with FFI. No XS required.
 
 =head1 VERSION
 
-version 0.19
+version 0.20
 
 =head1 SYNOPSIS
 
@@ -707,6 +736,10 @@ mostly useful for finding functions in the standard C library, without
 having to know the name of the standard c library for your platform (as 
 it turns out it is different just about everywhere!).
 
+You may also use the L</find_lib> method as a shortcut:
+
+ $ffi->find_lib( lib => 'archive' );
+
 =head2 ignore_not_found
 
 [version 0.15]
@@ -858,6 +891,9 @@ Examples:
  $ffi->attach($name => \@argument_types => $return_type);
  $ffi->attach([$c_name => $perl_name] => \@argument_types => $return_type);
  $ffi->attach([$address => $perl_name] => \@argument_types => $return_type);
+ $ffi->attach($name => \@argument_types => $return_type, sub { ... });
+ $ffi->attach([$c_name => $perl_name] => \@argument_types => $return_type, sub { ... });
+ $ffi->attach([$address => $perl_name] => \@argument_types => $return_type, sub { ... });
 
 Find and attach a C function as a real live Perl xsub.  The advantage of 
 attaching a function over using the L<function|/function> 
@@ -879,6 +915,24 @@ Examples:
  my $string1 = my_function_name($int);
  my $string2 = my_perl_function_name($int);
 
+[version 0.20]
+
+If the last argument is a code reference, then it will be used as a wrapper
+around the attached xsub.  The first argument to the wrapper will be the
+inner xsub.  This can be used if you need to verify/modify input/output
+data.
+
+Examples:
+
+ $ffi->attach('my_function', ['int', 'string'] => 'string', sub {
+   my($my_function_xsub, $integer, $string) = @_;
+   $integer++;
+   $string .= " and another thing";
+   my $return_string = $my_function->($integer, $string);
+   $return_string =~ /Belgium//; # HHGG remove profanity
+   $return_string;
+ });
+
 =head2 closure
 
  my $closure = $ffi->closure($coderef);
@@ -931,6 +985,19 @@ This function is not very fast, so you might want to save this value as
 a constant, particularly if you need the size in a loop with many 
 iterations.
 
+=head2 find_lib
+
+[version 0.20]
+
+ $ffi->find_lib( lib => $libname );
+
+This is just a shortcut for calling L<FFI::CheckLib#find_lib> and
+updating the L</lib> attribute appropriately.  Care should be taken
+though, as this method simply passes its arguments to
+L<FFI::CheckLib#find_lib>, so if your module or script is depending
+on a specific feature in L<FFI::CheckLib> then make sure that you
+update your prerequisites appropriately.
+
 =head2 find_symbol
 
  my $address = $ffi->find_symbol($name);
@@ -939,6 +1006,8 @@ Return the address of the given symbol (usually function).
 
 =head2 package
 
+[version 0.15]
+
  $ffi->package($package, $file); # usually __PACKAGE__ and __FILE__ can be used
  $ffi->package;                  # autodetect
 
@@ -1010,6 +1079,23 @@ friendly diagnostic letting the user know that the required library is
 missing, and reduce the number of bogus CPAN testers results that you 
 will get.
 
+Also in this example, we rename some of the functions when they are
+placed into Perl space to save typing:
+
+ attach [notify_notification_new => 'notify_new']
+   => [string,string,string]
+   => opaque;
+
+When you specify a list reference as the "name" of the function the
+first element is the symbol name as understood by the dynamic library.
+The second element is the name as it will be placed in Perl space.
+
+Later, when we call C<notify_new>:
+
+ my $n = notify_new('','','');
+
+We are really calling the C function C<notify_notification_new>.
+
 =head2 Allocating and freeing memory
 
  use FFI::Platypus::Declare;
@@ -1259,7 +1345,7 @@ implemented using FFI called L<ZMQ::FFI>.
  # the ArchiveWrite class that could be used for writing archive formats
  # supported by libarchive
  
- my $ffi = FFI::Platypus->new;
+ my $ffi = My::Platypus->new;
  $ffi->lib(FFI::CheckLib::find_lib_or_exit lib => 'archive');
  
  $ffi->custom_type(archive => {
@@ -1283,38 +1369,50 @@ implemented using FFI called L<ZMQ::FFI>.
    },
  });
  
+ package My::Platypus;
+ 
+ use base qw( FFI::Platypus );
+ 
+ sub find_symbol
+ {
+   my($self, $name) = @_;
+   my $prefix = lcfirst caller(2);
+   $prefix =~ s{([A-Z])}{"_" . lc $1}eg;
+   $self->SUPER::find_symbol(join '_', $prefix, $name);
+ }
+ 
  package Archive;
  
  # base class is "abstract" having no constructor or destructor
  
- $ffi->attach( [ archive_error_string => 'error_string' ] => ['archive'] => 'string' );
+ $ffi->attach( error_string => ['archive'] => 'string' );
  
  package ArchiveRead;
  
  our @ISA = qw( Archive );
  
- $ffi->attach( [ archive_read_new => 'new' ] => ['string'] => 'archive' );
- $ffi->attach( [ archive_read_free => 'DESTROY' ] => ['archive'] => 'void' );
- $ffi->attach( [ archive_read_support_filter_all => 'support_filter_all' ] => ['archive'] => 'int' );
- $ffi->attach( [ archive_read_support_format_all => 'support_format_all' ] => ['archive'] => 'int' );
- $ffi->attach( [ archive_read_open_filename => 'open_filename' ] => ['archive','string','size_t'] => 'int' );
- $ffi->attach( [ archive_read_next_header2 => 'next_header2' ] => ['archive', 'archive_entry' ] => 'int' );
- $ffi->attach( [ archive_read_data_skip => 'data_skip' ] => ['archive'] => 'int' );
+ $ffi->attach( new                   => ['string']                    => 'archive' );
+ $ffi->attach( [ free => 'DESTROY' ] => ['archive']                   => 'void' );
+ $ffi->attach( support_filter_all    => ['archive']                   => 'int' );
+ $ffi->attach( support_format_all    => ['archive']                   => 'int' );
+ $ffi->attach( open_filename         => ['archive','string','size_t'] => 'int' );
+ $ffi->attach( next_header2          => ['archive', 'archive_entry' ] => 'int' );
+ $ffi->attach( data_skip             => ['archive']                   => 'int' );
  # ... define additional read methods
  
  package ArchiveWrite;
  
  our @ISA = qw( Archive );
  
- $ffi->attach( [ archive_write_new => 'new' ] => ['string'] => 'archive' );
- $ffi->attach( [ archive_write_free => 'DESTROY' ] => ['archive'] => 'void' );
+ $ffi->attach( new                   => ['string'] => 'archive' );
+ $ffi->attach( [ free => 'DESTROY' ] => ['archive'] => 'void' );
  # ... define additional write methods
  
  package ArchiveEntry;
  
- $ffi->attach( [ archive_entry_new => 'new' ] => ['string'] => 'archive_entry' );
- $ffi->attach( [ archive_entry_free => 'DESTROY' ] => ['archive_entry'] => 'void' );
- $ffi->attach( [ archive_entry_pathname => 'pathname' ] => ['archive_entry'] => 'string' );
+ $ffi->attach( new => ['string']     => 'archive_entry' );
+ $ffi->attach( [ free => 'DESTROY' ] => ['archive_entry'] => 'void' );
+ $ffi->attach( pathname              => ['archive_entry'] => 'string' );
  # ... define additional entry methods
  
  package main;
@@ -1354,9 +1452,27 @@ inherited from and extended just like any Perl classes because of the
 way the custom types are implemented.  For more details on custom types 
 see L<FFI::Platypus::Type> and L<FFI::Platypus::API>.
 
+Another advanced feature of this example is that we extend the 
+L<FFI::Platypus> class to define our own find_symbol method that prefixes
+the symbol names depending on the class in which they are defined.
+This means we can do this when we define a method for Archive:
+
+ $ffi->attach( support_filter_all => ['archive'] => 'int' );
+
+Rather than this:
+
+ $ffi->attach(
+   [ archive_read_support_filter_all => 'support_read_filter_all' ] => 
+   ['archive'] => 'int' );
+ );
+
+If you didn't want to create an entire new class just for this little
+trick you could also use something like L<Object::Method> to extend
+C<find_symbol>.
+
 =head2 bzip2
 
- use FFI::Platypus;
+ use FFI::Platypus 0.20 (); # 0.20 required for using wrappers
  use FFI::CheckLib qw( find_lib_or_die );
  use FFI::Platypus::Buffer qw( scalar_to_buffer buffer_to_scalar );
  use FFI::Platypus::Memory qw( malloc free );
@@ -1365,7 +1481,7 @@ see L<FFI::Platypus::Type> and L<FFI::Platypus::API>.
  $ffi->lib(find_lib_or_die lib => 'bz2');
  
  $ffi->attach(
-   BZ2_bzBuffToBuffCompress => [
+   [ BZ2_bzBuffToBuffCompress => 'compress' ] => [
      'opaque',				# dest
      'unsigned int *',			# dest length
      'opaque',				# source
@@ -1374,10 +1490,21 @@ see L<FFI::Platypus::Type> and L<FFI::Platypus::API>.
      'int',				# verbosity
      'int',				# workFactor
    ] => 'int',
+   sub {
+     my $sub = shift;
+     my($source,$source_length) = scalar_to_buffer $_[0];
+     my $dest_length = int(length($source)*1.01) + 1 + 600;
+     my $dest = malloc $dest_length;
+     my $r = $sub->($dest, \$dest_length, $source, $source_length, 9, 0, 30);
+     die "bzip2 error $r" unless $r == 0;
+     my $compressed = buffer_to_scalar($dest, $dest_length);
+     free $dest;
+     $compressed;
+   },
  );
  
  $ffi->attach(
-   BZ2_bzBuffToBuffDecompress => [
+   [ BZ2_bzBuffToBuffDecompress => 'decompress' ] => [
      'opaque',				# dest
      'unsigned int *',			# dest length
      'opaque',				# source
@@ -1385,44 +1512,45 @@ see L<FFI::Platypus::Type> and L<FFI::Platypus::API>.
      'int',				# small
      'int',				# verbosity
    ] => 'int',
+   sub {
+     my $sub = shift;
+     my($source, $source_length) = scalar_to_buffer $_[0];
+     my $dest_length = $_[1];
+     my $dest = malloc $dest_length;
+     my $r = $sub->($dest, \$dest_length, $source, $source_length, 0, 0);
+     die "bzip2 error $r" unless $r == 0;
+     my $decompressed = buffer_to_scalar($dest, $dest_length);
+     free $dest;
+     $decompressed;
+   },
  );
  
- sub compress ($)
- {
-   my($source,$source_length) = scalar_to_buffer($_[0]);
-   my $dest_length = int(length($source)*1.01) + 1 + 600;
-   my $dest = malloc $dest_length;
-   my $r = BZ2_bzBuffToBuffCompress($dest, \$dest_length, $source, $source_length, 9, 0, 30);
-   die "bzip2 error $r" unless $r == 0;
-   my $compressed = buffer_to_scalar($dest, $dest_length);
-   free $dest;
-   $compressed;
- };
- 
- sub decompress ($$)
- {
-   my($source, $source_length) = scalar_to_buffer($_[0]);
-   my $dest_length = $_[1];
-   my $dest = malloc $dest_length;
-   my $r = BZ2_bzBuffToBuffDecompress($dest, \$dest_length, $source, $source_length, 0, 0);
-   die "bzip2 error $r" unless $r == 0;
-   my $decompressed = buffer_to_scalar($dest, $dest_length);
- }
- 
  my $original = "hello compression world\n";
- my $compressed = compress $original;
- print decompress $compressed, length $original;
+ my $compressed = compress($original);
+ print decompress($compressed, length $original);
 
 B<Discussion>: bzip2 is a compression library.  For simple one shot
 attempts at compression/decompression when you expect the original
 and the result to fit within memory it provides two convenience functions
 C<BZ2_bzBuffToBuffCompress> and C<BZ2_bzBuffToBuffDecompress>.
 
-The first four arguments of both of these functions are identical, and 
-represent two buffers.  For the destination buffer, the length is passed 
-in as an pointer to an integer.  The value of this integer going in is 
-the maximum size of the buffer (the amount of memory preallocated).  On 
-output, the bzip2 library updates it with the actual number of bytes used.
+The first four arguments of both of these C functions are identical, and 
+represent two buffers.  One buffer is the source, the second is the 
+destination.  For the destination, the length is passed in as a pointer 
+to an integer.  On input this integer is the size of the destination 
+buffer, and thus the maximum size of the compressed or decompressed 
+data.  When the function returns the actual size of compressed or 
+compressed data is stored in this integer.
+
+This is normal stuff for C, but in Perl our buffers are scalars and they 
+already know how large they are.  In this sort of situation, wrapping 
+the C function in some Perl code can make your interface a little more 
+Perl like.  In order to do this, just provide a code reference as the 
+last argument to the L</attach> method.  The first argument to this 
+wrapper will be a code reference to the C function.  The Perl arguments 
+will come in after that.  This allows you to modify / convert the 
+arguments to conform to the C API.  What ever value you return from the 
+wrapper function will be returned back to the original caller.
 
 =head1 SUPPORT
 
