@@ -5,7 +5,7 @@ use warnings;
 use FFI::Platypus;
 
 # ABSTRACT: Declarative interface to FFI::Platypus
-our $VERSION = '0.19'; # VERSION
+our $VERSION = '0.20'; # VERSION
 
 
 our $ffi    = {};
@@ -48,13 +48,30 @@ sub type_meta($)
 }
 
 
-sub attach ($$$;$)
+my $inner_counter = 0;
+
+sub attach ($$$;$$)
 {
   my($caller, $filename, $line) = caller;
+  my $wrapper;
+  $wrapper = pop if ref($_[-1]) eq 'CODE';
   my($name, $args, $ret, $proto) = @_;
   my($symbol_name, $perl_name) = ref $name ? (@$name) : ($name, $name);
   my $function = _ffi_object->function($symbol_name, $args, $ret);
-  $function->attach(join('::', $caller, $perl_name), "$filename:$line", $proto);
+  my $attach_name = $perl_name = join '::', $caller, $perl_name;
+  if($wrapper)
+  {
+    $attach_name = "FFI::Platypus::Declare::Inner::xsub$inner_counter";
+    $inner_counter++;
+  }
+  $function->attach($attach_name, "$filename:$line", $proto);
+  if($wrapper)
+  {
+    my $inner = \&{$attach_name};
+    no strict 'refs';
+    *{$perl_name} = sub { $wrapper->($inner, @_) };
+  }
+  ();
 }
 
 
@@ -150,7 +167,7 @@ FFI::Platypus::Declare - Declarative interface to FFI::Platypus
 
 =head1 VERSION
 
-version 0.19
+version 0.20
 
 =head1 SYNOPSIS
 
