@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 # ABSTRACT: Documentation and tools for using Platypus with the Assembly
-our $VERSION = '0.19'; # VERSION
+our $VERSION = '0.20'; # VERSION
 
 
 sub native_type_map
@@ -26,7 +26,7 @@ FFI::Platypus::Lang::ASM - Documentation and tools for using Platypus with the A
 
 =head1 VERSION
 
-version 0.19
+version 0.20
 
 =head1 SYNOPSIS
 
@@ -36,7 +36,11 @@ version 0.19
 
 =head1 DESCRIPTION
 
-Setting your lang to C<ASM> removes all non ffi types.  
+Setting your lang to C<ASM> includes no native type aliases, so types
+like C<int> or C<unsigned long> will not work.  You need to specify
+instead C<sint32> or C<sint64>.  Although intended for use with Assembly
+it could also be used for other languages if you did not want to use
+the normal C aliases for native types.
 
 This document will one day include information on bundling Assembly
 with your Perl / FFI / Platypus distribution.  Pull requests welcome!
@@ -45,11 +49,11 @@ with your Perl / FFI / Platypus distribution.  Pull requests welcome!
 
 =head2 native_type_map
 
- my $hashref = FFI::Platypus::Lang::C->native_type_map;
+ my $hashref = FFI::Platypus::Lang::ASM->native_type_map;
 
-This returns a hash reference containing the native aliases for the
-C programming languages.  That is the keys are native C types and the
-values are libffi native types.
+This returns an empty hash reference.  For other languages it returns
+a hash reference that defines the aliases for the types normally used
+for that language.
 
 =head1 SEE ALSO
 
